@@ -14,11 +14,14 @@ pub use tokio_rustls::client::TlsStream as TlsClientStream;
 pub use tokio_rustls::server::TlsStream as TlsServerStream;
 
 // ========== client ==========
+#[derive(Debug, Clone)]
 pub struct TlsClientConf {
     pub sni: String,
     pub insecure: bool,
+    pub early_data: bool,
 }
 
+#[derive(Clone)]
 pub struct TlsConnect<T> {
     conn: T,
     sni: ServerName,
@@ -27,9 +30,14 @@ pub struct TlsConnect<T> {
 
 impl<T> TlsConnect<T> {
     pub fn new(conn: T, conf: TlsClientConf) -> Self {
-        let sni: ServerName = conf.sni.as_str().try_into().expect("invalid DNS name");
+        let TlsClientConf {
+            sni,
+            insecure,
+            early_data,
+        } = conf;
+        let sni: ServerName = sni.as_str().try_into().expect("invalid DNS name");
 
-        let conf = if !conf.insecure {
+        let mut conf = if !insecure {
             ClientConfig::builder()
                 .with_safe_defaults()
                 .with_root_certificates(utils::firefox_roots())
@@ -41,6 +49,8 @@ impl<T> TlsConnect<T> {
                 .with_no_client_auth()
         };
 
+        conf.enable_early_data = early_data;
+
         Self {
             conn,
             sni,
@@ -68,12 +78,14 @@ where
 }
 
 // ========== server ==========
+#[derive(Debug, Clone)]
 pub struct TlsServerConf {
     pub crt: String,
     pub key: String,
     pub server_name: String,
 }
 
+#[derive(Clone)]
 pub struct TlsAccept<T> {
     lis: T,
     ac: TlsAcceptor,
